/*
Main code by Richard Visokey AD7C - www.ad7c.com
Revision 2.0 - November 6th, 2013
*/

// Include the library code
#include <LiquidCrystal.h>
#include <rotary.h>
#include <EEPROM.h>

//Setup some items
#define W_CLK 8   // Pin 8 - connect to AD9850 module word load clock pin (CLK)
#define FQ_UD 9   // Pin 9 - connect to freq update pin (FQ)
#define DATA 10   // Pin 10 - connect to serial data load pin (DATA)
#define RESET 11  // Pin 11 - connect to reset pin (RST) 
#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }
Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  Must be interrupt pins.
LiquidCrystal lcd(12, 13, 7, 6, 5, 4); // I used an odd pin combination because I need pin 2 and 3 for the interrupts.
int_fast32_t rx=1485000; // Starting frequency of EU VFO
int_fast32_t rx2=1; // variable to hold the updated frequency
int_fast32_t euh=1602000; // European High Frequency limit
int_fast32_t eul=531000;  // European Low Frequency limit
int_fast32_t ush=1700000; // US High Frequency limit
int_fast32_t usl=530000; // US Low Frequency limit
int EU_frequencymode = 1;
int_fast32_t increment = 9000; // starting VFO update increment in Hz.
int buttonstate = 0;
int audio = 0;
byte ones,tens,hundreds,thousands,tenthousands,hundredthousands,millions ;  //Placeholders
String freq; // string to hold the frequency
int_fast32_t timepassed = millis(); // int to hold the arduino millis since startup
int memstatus = 1;  // value to notify if memory is current or old. 0=old, 1=current.



void setup() {
  pinMode(A0,INPUT); // Connect to a button that goes to GND on push
  digitalWrite(A0,HIGH);
  lcd.begin(16, 2);
  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();
  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 
  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.
  lcd.setCursor(0,1);    
  lcd.print("Audio ");
   // Load the stored frequency  
  if (EEPROM.read(15) == 45) {
    EU_frequencymode = EEPROM.read(14);
    if (EU_frequencymode == 1) {
      freq = String(EEPROM.read(0))+String(EEPROM.read(1))+String(EEPROM.read(2))+String(EEPROM.read(3))+String(EEPROM.read(4))+String(EEPROM.read(5))+String(EEPROM.read(6));
      increment = 9000;
    } else {
      freq = String(EEPROM.read(7))+String(EEPROM.read(8))+String(EEPROM.read(9))+String(EEPROM.read(10))+String(EEPROM.read(11))+String(EEPROM.read(12))+String(EEPROM.read(13));
      increment = 10000;
    }
    rx = freq.toInt();  
  } else {
   EEPROM.write(0,1);
   EEPROM.write(1,4);
   EEPROM.write(2,8);
   EEPROM.write(3,5);
   EEPROM.write(4,0);       
   EEPROM.write(5,0);
   EEPROM.write(6,0);
   EEPROM.write(7,1);
   EEPROM.write(8,5);
   EEPROM.write(9,0);
   EEPROM.write(10,0);
   EEPROM.write(11,0);       
   EEPROM.write(12,0);
   EEPROM.write(13,0);
   EEPROM.write(14,EU_frequencymode);
   EEPROM.write(15,45);
  }
  sendFrequency(rx);
  byte M1[8] = {
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B11111,
  };
  byte M2[8] = {
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B11111,
    B11111,
  };
  byte M3[8] = {
    B00000,
    B00000,
    B00000,
    B00000,
    B00000,
    B11111,
    B11111,
    B11111,
  };
  byte M4[8] = {
    B00000,
    B00000,
    B00000,
    B00000,
    B11111,
    B11111,
    B11111,
    B11111,
  };
  byte M5[8] = {
    B00000,
    B00000,
    B00000,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
  };
  byte M6[8] = {
    B00000,
    B00000,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
  };
  byte M7[8] = {
    B00000,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
  };
  byte M8[8] = {
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
  };
  lcd.createChar(0, M1);
  lcd.createChar(1, M2);
  lcd.createChar(2, M3);
  lcd.createChar(3, M4);
  lcd.createChar(4, M5);
  lcd.createChar(5, M6);
  lcd.createChar(6, M7);
  lcd.createChar(7, M8);
}


void loop() {
  if (rx != rx2){    
        showFreq();
        sendFrequency(rx);
        rx2 = rx;
      }
      
  buttonstate = digitalRead(A0);
  if(buttonstate == LOW) {
    delay(50);
    if(buttonstate == LOW) { // software debounce
        setmode();        
    };
  };  

  // Write the frequency to memory if not stored and 2 seconds have passed since the last frequency change.
    if(memstatus == 0){   
      if(timepassed+2000 < millis()){
        storeMEM();
      }
    }

  // Present analog information from Compressor on screen
  audio = analogRead(A1);
  lcd.setCursor(6,1);
  if (audio > 10) {
    lcd.write(byte(0));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(7,1);
  if (audio > 35) {
    lcd.write(byte(1));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(8,1);
  if (audio > 90) {
    lcd.write(byte(2));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(9,1);
  if (audio > 140) {
    lcd.write(byte(3));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(10,1);
  if (audio > 175) {
    lcd.write(byte(4));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(11,1);
  if (audio > 210) {
    lcd.write(byte(5));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(12,1);
  if (audio > 245) {
    lcd.write(byte(6));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(13,1);
  if (audio > 280) {
    lcd.write(byte(7));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(14,1);
  if (audio > 305) {
    lcd.write(byte(7));
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(15,1);
  if (audio > 330) {
    lcd.print("C");
  } else {
    lcd.print(" ");
  }
}


ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){rx=rx+increment;}
    else {rx=rx-increment;};
    if (EU_frequencymode == 1) {       
      if (rx > euh){rx=euh;}; // UPPER VFO LIMIT
      if (rx < eul){rx=eul;}; // LOWER VFO LIMIT
    } else {
      if (rx > ush){rx=ush;}; // UPPER VFO LIMIT
      if (rx < usl){rx=usl;}; // LOWER VFO LIMIT
    }
    rx2=1;
  }
}



// frequency calc from datasheet page 8 = <sys clock> * <frequency tuning word>/2^32
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on 9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for 9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}
// transfers a byte, a bit at a time, LSB first to the 9850 via serial DATA line
void tfr_byte(byte data)
{
  for (int i=0; i<8; i++, data>>=1) {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}

void setmode(){
  if (EU_frequencymode == 1) {
    showFreq(); // this will recalc all frequency bytes
    storeMEM(); // store current EU frequency
    freq = String(EEPROM.read(7))+String(EEPROM.read(8))+String(EEPROM.read(9))+String(EEPROM.read(10))+String(EEPROM.read(11))+String(EEPROM.read(12))+String(EEPROM.read(13));
    increment = 10000;
    rx = freq.toInt();
    showFreq(); // show new frequency
    sendFrequency(rx);
    EU_frequencymode = 0;
    EEPROM.write(14,EU_frequencymode); // update frequency mode in memory
  } else {
    showFreq(); // this will recalc all frequency bytes
    storeMEM(); // store current EU frequency
    freq = String(EEPROM.read(0))+String(EEPROM.read(1))+String(EEPROM.read(2))+String(EEPROM.read(3))+String(EEPROM.read(4))+String(EEPROM.read(5))+String(EEPROM.read(6));
    increment = 9000;
    rx = freq.toInt();
    showFreq(); // show new frequency
    sendFrequency(rx);
    EU_frequencymode = 1;
    EEPROM.write(14,EU_frequencymode); // update frequency mode in memory
  }
  while (digitalRead(A0) == LOW) {
    delay(50); // wait until button released
  }
  delay(250); // Extra timeout.
};

void showFreq(){
    millions = int(rx/1000000);
    hundredthousands = ((rx/100000)%10);
    tenthousands = ((rx/10000)%10);
    thousands = ((rx/1000)%10);
    hundreds = ((rx/100)%10);
    tens = ((rx/10)%10);
    ones = ((rx/1)%10);
    lcd.setCursor(0,0);
    lcd.print("                ");
   if (millions > 9){lcd.setCursor(1,0);}
   else{lcd.setCursor(2,0);}
    lcd.print(millions);
    lcd.print(".");
    lcd.print(hundredthousands);
    lcd.print(tenthousands);
    lcd.print(thousands);
    lcd.print(".");
    lcd.print(hundreds);
    lcd.print(tens);
    lcd.print(ones);
    lcd.print(" MHz ");
    timepassed = millis();
    memstatus = 0; // Trigger memory write
};

void storeMEM(){
  //Write each frequency section to a EPROM slot.  Yes, it's cheating but it works!
  if (EU_frequencymode == 1) {
   EEPROM.write(0,millions);
   EEPROM.write(1,hundredthousands);
   EEPROM.write(2,tenthousands);
   EEPROM.write(3,thousands);
   EEPROM.write(4,hundreds);       
   EEPROM.write(5,tens);
   EEPROM.write(6,ones);
  } else {
   EEPROM.write(7,millions);
   EEPROM.write(8,hundredthousands);
   EEPROM.write(9,tenthousands);
   EEPROM.write(10,thousands);
   EEPROM.write(11,hundreds);       
   EEPROM.write(12,tens);
   EEPROM.write(13,ones);
  }
  EEPROM.write(14,EU_frequencymode);
  EEPROM.write(15,45);
  memstatus = 1;  // Let program know memory has been written
};


